{
DESCRIPTION:
This unit tests StrLib.pas.

COPYRIGHT:
Copyright 2002 Rio Software, LLC.  All rights reserved.

HISTORY:
Created on 8/6/2002 by Colin Patrick Sarsfield & Andrew Cray Vogan
}

unit VStrLib;

interface

uses
  Classes, TestFrameWork, StrLib;

type
  TTestStrLib = class(TTestCase)
  private
  published
    procedure TestAnsiLowerCaseNonAcronyms;
    procedure TestDropLeadingZeros;
    procedure TestFloatToFixedStr;
    procedure TestForgivingStrToDate;
  end;

implementation

uses
  SysUtils;

{ TTestStrLib }

procedure TTestStrLib.TestAnsiLowerCaseNonAcronyms;
const
  TestCases: array[0..6, 0..1] of String = (
    ('I eat iMac''s for breakfast.', 'i eat iMac''s for breakfast.'),
    ('the .NET Architecture','the .NET architecture'),
    ('all Ph.D''s unite!','all ph.d''s unite!'),
    ('calculations by HeaTrans','calculations by HeaTrans'),
    ('Multiple ACRONYM Test (StrLib)','multiple ACRONYM test (StrLib)'),
    ('BAC-Net','BAC-Net'),
    ('','')
    );
var
  i: Integer;
begin
  for i := 0 to High(TestCases) do
    Check(AnsiLowerCaseNonAcronyms(TestCases[i, 0]) = TestCases[i, 1],
        Format('Failed TestAnsiLowerCaseNonAcronyms on test index %d; got %s, expected %s',
        [i, AnsiLowerCaseNonAcronyms(TestCases[i, 0]), TestCases[i, 1]]));
end;

procedure TTestStrLib.TestDropLeadingZeros;
begin
  CheckEquals('35', DropLeadingZeros('0035'));
  CheckEquals('.75', DropLeadingZeros('0.75'));
  CheckEquals('', DropLeadingZeros(''));
  CheckEquals('0', DropLeadingZeros('0'));
  CheckEquals('0', DropLeadingZeros('00'));
  CheckEquals('.', DropLeadingZeros('0.'));
  CheckEquals('.', DropLeadingZeros('.'));
  CheckEquals('.0', DropLeadingZeros('.0'));
end;

procedure TTestStrLib.TestFloatToFixedStr;

  procedure TestValues(Value: Extended; Precision: Integer; ExpectedStr: String);
  var
    ResultStr: String;
  begin
    ResultStr := FloatToFixedStr(Value, Precision);
    Check(ResultStr = ExpectedStr, Format('FloatToFixedStr(%g, %d) = %s <> %s',
        [Value, Precision, ResultStr, ExpectedStr]));
  end;
begin
  TestValues(-12e-10, 5, '-0.0000000012');
  TestValues(123, 7, '123');
  TestValues(0, 5, '0');
  TestValues(1.25e25, 5, '12500000000000000000000000');
  TestValues(5.55555555555555555555555, 3, '5.56');
  TestValues(1020, 18, '1020');
  TestValues(0.00015, 18, '0.00015');
  TestValues(-123, 7, '-123');
  TestValues(-0, 5, '0');
  TestValues(-1.25e25, 5, '-12500000000000000000000000');
  TestValues(-5.55555555555555555555555, 3, '-5.56');
  TestValues(-1020, 18, '-1020');
  TestValues(-0.00015, 18, '-0.00015');
end;

procedure TTestStrLib.TestForgivingStrToDate;
var
  CurrentYear, CurrentMonth, CurrentDay: Word;

  procedure CheckValues(TestStr: String; Year, Month, Day: Word);
  var
    ResultDate: TDateTime;
  begin
    ResultDate := ForgivingStrToDate(TestStr);
    Check(ResultDate = EncodeDate(Year, Month, Day),
        TestStr + ' ---> ' + FormatDateTime('ddddd', ResultDate));
  end;
begin
  DecodeDate(Date, CurrentYear, CurrentMonth, CurrentDay);

  CheckValues('May 2', CurrentYear, 5, 2);
  CheckValues('2May03', 2003, 5, 2);
  CheckValues('9/11/2002', 2002, 9, 11);
  CheckValues('9 11 02', 2002, 9, 11);
  CheckValues('2/Jun/02', 2002, 6, 2);
  CheckValues('Jan', CurrentYear, 1, 1);
  CheckValues('2003.Feb.4', 2003, 2, 4);
  CheckValues('2003.Feb.30', 2003, 2, 28);
  CheckValues('2000.Feb.29', 2000, 2, 29);
  CheckValues('2100.Feb.29', 2100, 2, 28);
  CheckValues('13-1-04', 2004, 1, 13);
  CheckValues('12-17-74', 1974, 12, 17);
  CheckValues('April 16', CurrentYear, 4, 16);
  CheckValues('April 16th', CurrentYear, 4, 16);
  CheckValues('August 7, 2001', 2001, 8, 7);
end;

initialization
  TestFramework.RegisterTest(TTestStrLib.Suite);
end.

